
def cmd_infos():
    cmd_info = ""
    cmd_info += "init_bulk".lower() + "\n"
    cmd_info += "you could use this method to do relax, super cell, scale, pertub and aimd\n"
    cmd_info += "example: PWact init_bulk init_param.json machine.json\n\n"
    
    # cmd_info += "int_surface".lower() + "\n"
    # cmd_info += ""
    cmd_info += "run".lower() + "\n"
    cmd_info += "you could use this method to do active learning sampling\n"
    cmd_info += "example: PWact run param.json machine.json\n\n"
    # cmd_info += "init_json".lower() + "\n"

    # cmd_info += "run_json".lower() + "\n"

    cmd_info += "to_pwdata".lower() + "\n"
    cmd_info += "you could use this method to change outcars or movements to pwdata format.\nFor more detail for this command, you could use 'PWact to_pwdata -h'\n\n"

    cmd_info += "gather_pwdata".lower() + "\n"
    cmd_info += "you could use this method to extract pwdatas after active learing done.\nFor more detail for this command, you could use 'PWact gather_pwdata -h'\n\n"
    
    cmd_info += "\n\n"
    cmd_info += "examples of pwact are located at the dir 'pwact/example'\n"
    cmd_info += "you can alse download them from github link:\n"
    cmd_info += "https://github.com/LonxunQuantum/PWact/tree/main/pwact/example\n\n"
    cmd_info += "or download from BaiduNetdisk included the calculation results of examples:\n"
    cmd_info += "https://pan.baidu.com/s/14E0u_7cpntiBZgg-C1S5XA?pwd=pwmt\n\n"

    return cmd_info


"""
    realx dor strcuture
    --/work_path/
    ------------/relax
    -----------------/*_init_config/*_init_.config, etot.input, pseudo files, final.config, mvms_relax
    ------------/super_cell_scale
    -----------------/*_init_config/scale.config, from final.config or *_init_.config
    ------------/perturb
    --------------------/*_init_config/scale.config or final.config
    ----------------------------------/structures/*.config_perturb
    ------------/AIMD
    -----------------/*_init_config/*_config_dir the config file from pertub or *_init_.config
    -------------------------------/etot.input, atom.config, ...
    -----------/result
    -----------------/*_init_config/relax_mvm_*_init_config if need; aimd_mvm_*_init_config if need
    
"""
import os

from pwact.active_learning.user_input.resource import Resource
from pwact.active_learning.user_input.init_bulk_input import InitBulkParam
from pwact.active_learning.init_bulk.duplicate_scale import get_config_files_with_order

from pwact.utils.constant import PWMAT, INIT_BULK, TEMP_STRUCTURE, SLURM_OUT, DFT_STYLE
from pwact.active_learning.slurm import SlurmJob, Mission
from pwact.utils.slurm_script import get_slurm_job_run_info, split_job_for_group, set_slurm_script_content
    
from pwact.utils.file_operation import write_to_file, link_file, search_files
from pwact.utils.app_lib.common import link_pseudo_by_atom, set_input_script
from pwact.data_format.configop import save_config, get_atom_type, load_config

class Relabel(object):
    def __init__(self, resource: Resource, input_param:InitBulkParam):
        self.resource = resource
        self.input_param = input_param
        self.init_configs = self.input_param.sys_config
        self.relax_dir = os.path.join(self.input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.relax)
        self.super_cell_scale_dir = os.path.join(self.input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.super_cell_scale)
        self.pertub_dir = os.path.join(self.input_param.root_dir,TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.pertub)
        self.aimd_dir = os.path.join(self.input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.aimd)
        self.real_aimd_dir = os.path.join(self.input_param.root_dir, INIT_BULK.aimd)
        
        self.scf_dir = os.path.join(self.input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.scf)
        self.real_scf_dir = os.path.join(self.input_param.root_dir, INIT_BULK.scf)
       
    def make_scf_work(self):
        scf_paths = []
        use_dftb = False
        for init_config in self.init_configs:
            if init_config.scf is False:
                continue
            init_config_name = "init_config_{}".format(init_config.config_index)
            #1. read construtures from aimd dir

            #2. set relabel dir 
            #       read trajs from ./aimd/init_config_0/relax/0_aimd/
            #       make scf dir ./relabel/init_config_0/relax/0_aimd/10-scf/files
            traj_list = search_files(os.path.join(self.aimd_dir, init_config_name), "*/*aimd")
            for traj_dir in traj_list:
                scf_dir = os.path.join(self.scf_dir, init_config_name, \
                    os.path.basename(os.path.dirname(traj_dir)),\
                    os.path.basename(traj_dir))

                traj_file_name = DFT_STYLE.get_aimd_config(self.resource.dft_style)

                scf_lsit = self.make_scf_file(
                    scf_dir      =scf_dir, 
                    traj_file    =os.path.join(traj_dir, traj_file_name), 
                    traj_format  =DFT_STYLE.get_format_by_postfix(traj_file_name), 
                    interval     = self.input_param.interval,
                    target_format=DFT_STYLE.get_pwdata_format(self.input_param.scf_style, is_cp2k_coord=True),
                    input_file   =init_config.scf_input_file,
                    kspacing     =init_config.scf_kspacing, 
                    flag_symm    =init_config.scf_flag_symm,
                    is_dftb      = False,
                    in_skf       =None)

                scf_paths.extend(scf_lsit)
        # make slurm script and slurm job
        self.make_scf_slurm_job_files(scf_paths, use_dftb)
    
    def check_work_done(self):
        slurm_remain, slurm_success = get_slurm_job_run_info(self.scf_dir, \
            job_patten="*-{}".format(INIT_BULK.scf_job), \
            tag_patten="*-{}".format(INIT_BULK.scf_tag))
        slurm_done = True if len(slurm_remain) == 0 and len(slurm_success) > 0 else False # len(slurm_remain) > 0 exist slurm jobs need to do
        return slurm_done
            
    def do_scf_jobs(self):
        mission = Mission()
        slurm_remain, slurm_success = get_slurm_job_run_info(self.scf_dir, \
            job_patten="*-{}".format(INIT_BULK.scf_job), \
            tag_patten="*-{}".format(INIT_BULK.scf_tag))
        slurm_done = True if len(slurm_remain) == 0 and len(slurm_success) > 0 else False # len(slurm_remain) > 0 exist slurm jobs need to do
        if slurm_done is False:
            #recover slurm jobs
            if len(slurm_remain) > 0:
                print("Run these relabel Jobs:\n")
                print(slurm_remain)
                for i, script_path in enumerate(slurm_remain):
                    slurm_job = SlurmJob()
                    tag_name = "{}-{}".format(os.path.basename(script_path).split('-')[0].strip(), INIT_BULK.scf_tag)
                    tag = os.path.join(os.path.dirname(script_path),tag_name)
                    slurm_job.set_tag(tag)
                    slurm_job.set_cmd(script_path)
                    mission.add_job(slurm_job)

            if len(mission.job_list) > 0:
                mission.commit_jobs()
                mission.check_running_job()
                mission.all_job_finished(error_type=SLURM_OUT.dft_out)
                # mission.move_slurm_log_to_slurm_work_dir()

    '''
    description: 
        input_file is scf input control file, for vasp is incar, for pwmat is etot.input
    return {*}
    author: wuxingxing
    '''    
    def make_scf_file(self, scf_dir:str, traj_file:str, traj_format:str, interval:int, target_format:str, \
                input_file:str, kspacing:float=None, flag_symm:int=None, is_dftb:bool=False, in_skf:str=None):
        config = load_config(format=traj_format, config=traj_file)
        index_list = list(range(0, len(config), interval))
        scf_lsit = []
        for index in index_list:
            save_dir = os.path.join(scf_dir, "{}-{}".format(index, INIT_BULK.scf))
            if not os.path.exists(save_dir):
                os.makedirs(save_dir)
            #1. set config file
            target_config = save_config(config=config[index], 
                                        input_format=traj_format,# or None, the same
                                        wrap = False, 
                                        direct = True, 
                                        sort = True, 
                                        save_format=target_format, 
                                        save_path=save_dir, 
                                        save_name=DFT_STYLE.get_normal_config(self.input_param.scf_style))

            atom_type_list, _ = get_atom_type(config[index])
            #2. set pseudo files
            # if not is_dftb:
            pseudo_names = link_pseudo_by_atom(
                pseudo_list     = self.input_param.dft_input.scf_pseudo, 
                target_dir      = save_dir, 
                atom_order      = atom_type_list, 
                dft_style       = self.resource.scf_style,
                basis_set_file  =self.input_param.dft_input.basis_set_file,
                potential_file  =self.input_param.dft_input.potential_file
                )
            # else:
            #     # link in.skf path to aimd dir
            #     pseudo_names = []
            #     target_dir = os.path.join(aimd_dir, PWMAT.in_skf)
            #     link_file(in_skf, target_dir)
            #3. make dft input file
            set_input_script(
                input_file=input_file,
                config=target_config,
                dft_style=self.resource.scf_style,
                kspacing=kspacing, 
                flag_symm=flag_symm, 
                save_dir = save_dir,
                pseudo_names=pseudo_names,
                basis_set_file_name=self.input_param.dft_input.basis_set_file,# these for cp2k
                potential_file_name=self.input_param.dft_input.potential_file
            )
            scf_lsit.append(save_dir)
        return scf_lsit

    def make_scf_slurm_job_files(self, scf_dir_list:list[str],use_dftb: bool=False):
        group_list = split_job_for_group(self.resource.scf_resource.group_size, scf_dir_list, self.resource.scf_resource.parallel_num)
        for group_index, group in enumerate(group_list):
            if group[0] == "NONE":
                continue
            jobname = "scf{}".format(group_index)
            tag_name = "{}-{}".format(group_index, INIT_BULK.scf_tag)
            tag = os.path.join(self.scf_dir, tag_name)
            run_cmd = self.resource.scf_resource.command
            group_slurm_script = set_slurm_script_content(gpu_per_node=self.resource.scf_resource.gpu_per_node, 
                number_node = self.resource.scf_resource.number_node, 
                cpu_per_node = self.resource.scf_resource.cpu_per_node,
                queue_name = self.resource.scf_resource.queue_name,
                custom_flags = self.resource.scf_resource.custom_flags,
                env_script = self.resource.scf_resource.env_script,
                job_name = jobname,
                run_cmd_template = run_cmd,
                group = group,
                job_tag = tag,
                task_tag = INIT_BULK.scf_tag, 
                task_tag_faild = INIT_BULK.scf_tag_failed,
                parallel_num=self.resource.scf_resource.parallel_num,
                check_type=self.resource.scf_style
                )
            slurm_script_name = "{}-{}".format(group_index, INIT_BULK.scf_job)
            slurm_job_file =  os.path.join(self.scf_dir, slurm_script_name)
            write_to_file(slurm_job_file, group_slurm_script, "w")

    def do_post_process(self):
        if os.path.exists(self.scf_dir):
            link_file(self.scf_dir, self.real_scf_dir)



import os

from pwact.active_learning.init_bulk.relax import Relax
from pwact.active_learning.init_bulk.duplicate_scale import do_pertub_work, do_post_pertub, pertub_done, \
    duplicate_scale_done, duplicate_scale, do_post_duplicate_scale
from pwact.active_learning.init_bulk.aimd import AIMD
from pwact.active_learning.init_bulk.relabel import Relabel
from pwact.active_learning.user_input.init_bulk_input import InitBulkParam
from pwact.active_learning.user_input.resource import Resource

from pwact.utils.constant import INIT_BULK, DFT_STYLE, TEMP_STRUCTURE
from pwact.utils.file_operation import copy_file, copy_dir, search_files, del_file, del_file_list, write_to_file
from pwact.data_format.configop import extract_pwdata

def init_bulk_run(resource: Resource, input_param:InitBulkParam):
    #1. do relax
    if input_param.is_relax:
        relax = Relax(resource, input_param)
        if not relax.check_work_done():
            # make relax work dir
            relax.make_relax_work()
            # do relax jobs
            relax.do_relax_jobs()
            # do post process
        relax.do_post_process()
    # do super cell and scale
    if not duplicate_scale_done(input_param):
        duplicate_scale(resource, input_param)
        do_post_duplicate_scale(resource, input_param)

    # do pertub
    if not pertub_done(input_param):
        do_pertub_work(resource, input_param)
        do_post_pertub(resource, input_param)

    # do aimd
    if input_param.is_aimd:
        aimd = AIMD(resource, input_param)
        if not aimd.check_work_done():
            aimd.make_aimd_work()
            aimd.do_aimd_jobs()
            aimd.do_post_process()

    # do relabel
    if input_param.is_scf:
        relabel = Relabel(resource, input_param)
        if not relabel.check_work_done():
            relabel.make_scf_work()
            relabel.do_scf_jobs()
            relabel.do_post_process()
    do_collection(resource, input_param)
       
def do_collection(resource: Resource, input_param:InitBulkParam):
    init_configs = input_param.sys_config
    
    relax_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.relax)
    duplicate_scale_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.super_cell_scale) 
    pertub_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.pertub)
    aimd_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.aimd)
    collection_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.collection)
    
    real_collection_dir = os.path.join(input_param.root_dir, INIT_BULK.collection)
    real_relax_dir = os.path.join(input_param.root_dir, INIT_BULK.relax)
    real_duplicate_scale_dir = os.path.join(input_param.root_dir, INIT_BULK.super_cell_scale) 
    real_pertub_dir = os.path.join(input_param.root_dir, INIT_BULK.pertub)
    real_aimd_dir = os.path.join(input_param.root_dir, INIT_BULK.aimd)

    relabel_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir, INIT_BULK.scf)
    real_relabel_dir = os.path.join(input_param.root_dir, INIT_BULK.scf)
    
        
    for init_config in init_configs:
        init_config_name = "init_config_{}".format(init_config.config_index)
        
        collection_work_dir = os.path.join(collection_dir, init_config_name)
        if not os.path.exists(collection_work_dir):
            os.makedirs(collection_work_dir)
        #1. copy relax config file
        if init_config.relax:
            source_config = os.path.join(relax_dir, init_config_name, DFT_STYLE.get_relaxed_config(resource.dft_style))
            target_config = os.path.join(collection_work_dir, DFT_STYLE.get_relaxed_config(resource.dft_style))
            copy_file(source_config, target_config)
        #2. copy super cell and scaled config file
        config_template = "*{}".format(DFT_STYLE.get_postfix(resource.dft_style))
        source_file_list = search_files(os.path.join(duplicate_scale_dir, init_config_name), config_template)
        for file in source_file_list:
            target_file = os.path.join(collection_dir, init_config_name, os.path.basename(file))
            copy_file(file, target_file)
                    
        #3. copy perturb structure
        if init_config.perturb is not None:
            source_dir = search_files(os.path.join(pertub_dir, init_config_name), "*")
            for source in source_dir:
                # copy .../path/pertub/init_config_0/0.95_scale to .../path/collection/init_config_0/0.95_scale
                target_dir = os.path.join(collection_dir, init_config_name, "{}_{}".format(os.path.basename(source),INIT_BULK.pertub))
                copy_dir(source, target_dir, symlinks=False)
            
        #4. copy aimd result
        source_aimd = []
        if init_config.aimd is True:
            source_aimd = search_files(os.path.join(aimd_dir, init_config_name), "*/*{}/{}".format(INIT_BULK.aimd, DFT_STYLE.get_aimd_config(resource.dft_style)))
            if len(source_aimd) == 0:
                continue
            source_aimd = sorted(source_aimd)
            #5. convert the aimd files (for vasp is outcar, for pwmat is movement) to npy format
            extract_pwdata(data_list=source_aimd, 
                    data_format=DFT_STYLE.get_aimd_config_format(resource.dft_style),
                    datasets_path=os.path.join(collection_dir, init_config_name, INIT_BULK.npy_format_save_dir), 
                    train_valid_ratio=input_param.train_valid_ratio, 
                    data_shuffle=input_param.data_shuffle, 
                    merge_data=True,
                    interval=1
                )

        #6 convert relabel datas
        if init_config.scf:
            source_scf = search_files(os.path.join(relabel_dir, init_config_name),\
                 "*/*/*/{}".format(DFT_STYLE.get_aimd_config(resource.scf_style)))
                 # init/0-aimd/0-scf/OUTCAR
            if len(source_aimd) == 0:
                continue
            source_scf = sorted(source_scf, key=lambda x:int(os.path.basename(os.path.dirname(x)).split('-')[0]), reverse=False)
            #5. convert the aimd files (for vasp is outcar, for pwmat is movement) to npy format
            extract_pwdata(data_list=source_scf, 
                    data_format=DFT_STYLE.get_format_by_postfix(os.path.basename(source_scf[0])),
                    datasets_path=os.path.join(collection_dir, init_config_name, "scf_pwdata"), 
                    train_valid_ratio=input_param.train_valid_ratio, 
                    data_shuffle=input_param.data_shuffle, 
                    merge_data=True,
                    interval=1
                )

    # delete link files
    del_file(real_relax_dir)
    del_file(real_duplicate_scale_dir)
    del_file(real_pertub_dir)
    del_file(real_aimd_dir)
    del_file(real_relabel_dir)

    # copy collection file to target
    copy_dir(collection_dir, real_collection_dir)
    if not input_param.reserve_work:
        # mv collection to real dir
        temp_work_dir = os.path.join(input_param.root_dir, TEMP_STRUCTURE.tmp_init_bulk_dir)
        del_file_list([temp_work_dir])

    # print the dir of pwdatas from aimd
    pwdatas = search_files(real_collection_dir, "*/{}".format(INIT_BULK.npy_format_save_dir))
    if len(pwdatas) > 0:
        pwdatas = sorted(pwdatas)
        result_lines = ["\"{}\",".format(_) for _ in pwdatas]
        result_lines = "\n".join(result_lines)
        # result_lines = result_lines[:-1] # Filter the last ','
        result_save_path = os.path.join(real_collection_dir, INIT_BULK.npy_format_name)
        write_to_file(result_save_path, result_lines, mode='w')
    
    # print the dir of relabel_pwdatas from relabel
    relebel_datas = search_files(real_collection_dir, "*/{}".format("scf_pwdata"))
    if len(relebel_datas) > 0:
        pwdatas = sorted(relebel_datas)
        result_lines = ["\"{}\",".format(_) for _ in pwdatas]
        result_lines = "\n".join(result_lines)
        # result_lines = result_lines[:-1] # Filter the last ','
        result_save_path = os.path.join(real_collection_dir, "scf_pwdata")
        write_to_file(result_save_path, result_lines, mode='w')
    


import subprocess
import pkg_resources
def check_envs():
    # for pwmat
    pass
    # check pwdata
    # try:
    #     package_version = pkg_resources.get_distribution('pwdata').version
    #     if pkg_resources.parse_version(min_version) <= pkg_resources.parse_version(package_version) <= pkg_resources.parse_version(max_version):
    #         print(f"{package_name} version {package_version} is within the required range [{min_version}, {max_version}].")
    #         return True
    #     else:
    #         print(f"{package_name} version {package_version} is NOT within the required range [{min_version}, {max_version}].")
    #         return False

    # check PWMLFF???
